@@ -20,6 +20,7 @@ from gettext import gettext as _
 import tempfile
 import os
 
+import gobject
 import gconf
 import dbus
 import gio
@@ -165,6 +166,13 @@ class ActivityInviteButton(BaseInviteButton):
 
     def _launch(self):
         """Join the activity in the invite."""
+
+        shell_model = shell.get_model()
+        activity = shell_model.get_activity_by_id(self._activity_model.get_id())
+        if activity:
+            activity.get_window().activate(gtk.get_current_event_time())
+            return
+
         registry = bundleregistry.get_registry()
         bundle = registry.get_bundle(self._bundle_id)
 
@@ -429,8 +437,12 @@ class ActivitiesTray(HTray):
 class BaseTransferButton(ToolButton):
     """Button with a notification attached
     """
-    def __init__(self):
+    def __init__(self, file_transfer):
         ToolButton.__init__(self)
+
+        self.file_transfer = file_transfer
+        file_transfer.connect('notify::state', self.__notify_state_cb)
+
         icon = Icon()
         self.props.icon_widget = icon
         icon.show()
@@ -445,18 +457,31 @@ class BaseTransferButton(ToolButton):
             frame.remove_notification(self.notif_icon)
             self.notif_icon = None
 
+    def remove(self):
+        frame = jarabe.frame.get_view()
+        frame.remove_notification(self.notif_icon)
+        self.props.parent.remove(self)
+
+    def __notify_state_cb(self, file_transfer, pspec):
+        logging.debug('_update state: %r %r' % (file_transfer.props.state,
+                file_transfer.reason_last_change))
+        if file_transfer.props.state == filetransfer.FT_STATE_CANCELLED:
+            if file_transfer.reason_last_change == \
+               filetransfer.FT_REASON_LOCAL_STOPPED:
+                self.remove()
+
 class IncomingTransferButton(BaseTransferButton):
     """UI element representing an ongoing incoming file transfer
     """
     def __init__(self, file_transfer):
-        BaseTransferButton.__init__(self)
+        BaseTransferButton.__init__(self, file_transfer)
 
         self._object_id = None
         self._metadata = {}
-        self._file_transfer = file_transfer
-        self._file_transfer.connect('notify::state', self.__notify_state_cb)
-        self._file_transfer.connect('notify::transferred-bytes',
-                                    self.__notify_transferred_bytes_cb)
+
+        file_transfer.connect('notify::state', self.__notify_state_cb)
+        file_transfer.connect('notify::transferred-bytes',
+                              self.__notify_transferred_bytes_cb)
 
         icons = gio.content_type_get_icon(file_transfer.mime_type).props.names
         icons.append('application-octet-stream')
@@ -477,7 +502,8 @@ class IncomingTransferButton(BaseTransferButton):
                                gtk.CORNER_TOP_LEFT)
 
     def create_palette(self):
-        palette = IncomingTransferPalette(self._file_transfer)
+        palette = IncomingTransferPalette(self.file_transfer)
+        palette.connect('dismiss-clicked', self.__dismiss_clicked_cb)
         palette.props.invoker = FrameWidgetInvoker(self)
         palette.set_group_id('frame')
         return palette
@@ -544,13 +570,14 @@ class IncomingTransferButton(BaseTransferButton):
     def __error_handler_cb(self, error):
         logging.debug('__error_handler_cb %r %s' % (self._object_id, error))
 
+    def __dismiss_clicked_cb(self, palette):
+        self.remove()
+
 class OutgoingTransferButton(BaseTransferButton):
     """UI element representing an ongoing outgoing file transfer
     """
     def __init__(self, file_transfer):
-        BaseTransferButton.__init__(self)
-
-        self._file_transfer = file_transfer
+        BaseTransferButton.__init__(self, file_transfer)
 
         icons = gio.content_type_get_icon(file_transfer.mime_type).props.names
         icons.append('application-octet-stream')
@@ -572,14 +599,25 @@ class OutgoingTransferButton(BaseTransferButton):
                                gtk.CORNER_TOP_LEFT)
 
     def create_palette(self):
-        palette = OutgoingTransferPalette(self._file_transfer)
+        palette = OutgoingTransferPalette(self.file_transfer)
+        palette.connect('dismiss-clicked', self.__dismiss_clicked_cb)
         palette.props.invoker = FrameWidgetInvoker(self)
         palette.set_group_id('frame')
         return palette
 
+    def __dismiss_clicked_cb(self, palette):
+        self.remove()
+
 class BaseTransferPalette(Palette):
     """Base palette class for frame or notification icon for file transfers
     """
+    __gtype_name__ = "SugarBaseTransferPalette"
+
+    __gsignals__ = {
+        'dismiss-clicked':          (gobject.SIGNAL_RUN_FIRST,
+                                     gobject.TYPE_NONE, ([])),
+    }
+
     def __init__(self, file_transfer):
         Palette.__init__(self, file_transfer.title)
 
@@ -635,6 +673,7 @@ class BaseTransferPalette(Palette):
 class IncomingTransferPalette(BaseTransferPalette):
     """Palette for frame or notification icon for incoming file transfers
     """
+    __gtype_name__ = "SugarIncomingTransferPalette"
     def __init__(self, file_transfer):
         BaseTransferPalette.__init__(self, file_transfer)
 
@@ -704,10 +743,21 @@ class IncomingTransferPalette(BaseTransferPalette):
             self.update_progress()
 
         elif self.file_transfer.props.state == filetransfer.FT_STATE_COMPLETED:
-            # TODO: What to do here?
+
+            for item in self.menu.get_children():
+                self.menu.remove(item)
+
+            menu_item = MenuItem(_('Dismiss'), icon_name='dialog-cancel')
+            menu_item.connect('activate', self.__dismiss_activate_cb)
+            self.menu.append(menu_item)
+            menu_item.show()
+
             self.update_progress()
         elif self.file_transfer.props.state == filetransfer.FT_STATE_CANCELLED:
 
+            for item in self.menu.get_children():
+                self.menu.remove(item)
+
             menu_item = MenuItem(_('Resume'), icon_name='dialog-cancel')
             menu_item.connect('activate', self.__resume_activate_cb)
             self.menu.append(menu_item)
@@ -732,7 +782,7 @@ class IncomingTransferPalette(BaseTransferPalette):
         return file_name
 
     def __decline_activate_cb(self, menu_item):
-        self.file_transfer.decline()
+        self.file_transfer.cancel()
 
     def __cancel_activate_cb(self, menu_item):
         self.file_transfer.cancel()
@@ -740,9 +790,14 @@ class IncomingTransferPalette(BaseTransferPalette):
     def __resume_activate_cb(self, menu_item):
         self.file_transfer.resume()
 
+    def __dismiss_activate_cb(self, menu_item):
+        self.emit('dismiss-clicked')
+
 class OutgoingTransferPalette(BaseTransferPalette):
     """Palette for frame or notification icon for outgoing file transfers
     """
+    __gtype_name__ = "SugarOutgoingTransferPalette"
+
     def __init__(self, file_transfer):
         BaseTransferPalette.__init__(self, file_transfer)
 
@@ -811,12 +866,20 @@ class OutgoingTransferPalette(BaseTransferPalette):
             self.update_progress()
 
         elif self.file_transfer.props.state == filetransfer.FT_STATE_COMPLETED:
-            # TODO: What to do here?
-            self.update_progress()
-        elif self.file_transfer.props.state == filetransfer.FT_STATE_CANCELLED:
-            # TODO: What to do here?
+
+            for item in self.menu.get_children():
+                self.menu.remove(item)
+
+            menu_item = MenuItem(_('Dismiss'), icon_name='dialog-cancel')
+            menu_item.connect('activate', self.__dismiss_activate_cb)
+            self.menu.append(menu_item)
+            menu_item.show()
+
             self.update_progress()
 
     def __cancel_activate_cb(self, menu_item):
         self.file_transfer.cancel()
 
+    def __dismiss_activate_cb(self, menu_item):
+        self.emit('dismiss-clicked')
+
