@@ -77,7 +77,7 @@ class Activity(gobject.GObject):
         self._activity_id = activity_id
         self._activity_info = activity_info
         self._launch_time = time.time()
-        self._launching = False
+        self._launching = True
 
         if window is not None:
             self.set_window(window)
@@ -169,7 +169,10 @@ class Activity(gobject.GObject):
 
     def get_xid(self):
         """Retrieve the X-windows ID of our root window"""
-        return self._window.get_xid()
+        if self._window is not None:
+            return self._window.get_xid()
+        else:
+            return None
 
     def get_window(self):
         """Retrieve the X-windows root window of this application
@@ -464,16 +467,17 @@ class ShellModel(gobject.GObject):
                 home_activity.props.launching = False
                 self.emit('launch-completed', home_activity)
 
-            startup_time = time.time() - home_activity.get_launch_time()
-            logging.debug('%s launched in %f seconds.' %
-                               (home_activity.get_type(), startup_time))
+                startup_time = time.time() - home_activity.get_launch_time()
+                logging.debug('%s launched in %f seconds.' %
+                                   (home_activity.get_type(), startup_time))
 
             if self._active_activity is None:
                 self._set_active_activity(home_activity)
 
     def _window_closed_cb(self, screen, window):
         if window.get_window_type() == wnck.WINDOW_NORMAL:
-            self._remove_activity_by_xid(window.get_xid())
+            if self._get_activity_by_xid(window.get_xid()) is not None:
+                self._remove_activity_by_xid(window.get_xid())
 
     def _get_activity_by_xid(self, xid):
         for home_activity in self._activities:
